--[[
  DOM:Lava Cooling for Mineteste - Rewrite (Lavacooling)
  The original code of the "Lavacooling" is from ...
  
  License: GPLv3

  Version: 20130416
]]--


lavacooling = {}

modpath=minetest.get_modpath("lavacooling")

dofile(modpath.."/materiais.lua")
dofile(modpath.."/criacoes.lua")
dofile(modpath.."/funcoes.lua")

-- Para compatibilidade com versões anteriores.
minetest.register_alias("lavacooling:obsidian","default:obsidian")
minetest.register_alias("lavacooling:obsidian_shard","default:obsidian_shard")

-- Usada para verificar a chance de aquecimento, resfriamento, taxa de conversão do contato de lava com água e chance de minérios.
-- Chance funciona invertida, 1 para 100% e valores maiores para percentuais menores.  Provavelmente chance de 1/n.
-- Valores como 500 acontecem muitas vezes, adicionando valores aleatórios...
lavacooling.LAVACOOLING_ATIVO = 1
lavacooling.CHANCE_DE_ESFRIAR = 100
lavacooling.CHANCE_DE_DERRETER = 200
lavacooling.CHANCE_LAVA_AGUA = 30
lavacooling.CHANCE_PEDRA = 300

lavacooling.COMPARATIVO_PARA_METAIS = 85 --15%

--Para valores internamente calculados de 1 a 100, se maior ou igual ao comparativo.
lavacooling.COMPARATIVO_PARA_METAIS_RAROS = 95 --5%


lavacooling.METAIS = {}
lavacooling.METAIS_RAROS = {}

--Sobrepõe as rotinas cool_lava_flowing e cool_lava_source por versões diferenciadas que desativam estas quando o módulo esta ativado.
default.cool_lava_flowing = lavacooling.cool_lava_flowing_disable
default.cool_lava_source = lavacooling.cool_lava_source_disable

--[DOM] Rotina alterada para permitir que lava esteja na diagonal de fontes de água.
--ABMs
minetest.register_abm ({
--  nodenames = {"default:lava_source", "default:lava_flowing"},
  nodenames = {"default:lava_flowing"},
  neighbors = {"default:water_source", "default:water_flowing", "default:water"},
  interval = 1.0,
  chance = lavacooling.CHANCE_LAVA_AGUA,
  action = function (pos)
    --Sai se rotina estiver desativada.
    if lavacooling.LAVACOOLING_ATIVO==0 then return end

    --Só torna a lava em pedra se a fonte de ar estiver com coincidência de dois eixos, senão ignora.
    --Pegando os 6 pontos chave de contato direto:

    local a = DOM_vizinhos(pos)

    if a.total_agua > 0 then
      if a.total_lava > 2 then
        minetest.env: add_node(pos, {name = "lavacooling:moltenrock"})
      else
        minetest.env: add_node(pos, {name = "default:obsidian"})
      end
    end
  end
})

-- Processo de resfriamento da pedra mole pela ação da água.
minetest.register_abm ({
  nodenames = {"lavacooling:moltenrock", "lavacooling:moltenrock_cold"},
  neighbors = {"default:water_source", "default:water_flowing", "default:water"},
  interval = 1.0,
  chance = 1,
  action = function (pos)
--DOM_inspeciona_r("Inspeção.")
    --Sai se rotina estiver desativada.
    if lavacooling.LAVACOOLING_ATIVO==0 then return end

    --Só torna a pedra mole em pedra se a fonte de água estiver com coincidência de dois eixos, senão ignora.
    --Pegando os 6 pontos chave de contato direto:
    local a = DOM_vizinhos(pos)
    local tipo =  minetest.env:get_node(pos).name

    if a.total_agua >= 1 then
      if a.n[2]=="air" and (a.n[1]=="default:lava_source" or a.n[1]=="default:lava_flowing") and tipo == "lavacooling:moltenrock" then
        minetest.env: add_node (pos, {name = "lavacooling:pumice"})
      else
        if(tipo == "lavacooling:moltenrock_cold") then
          local gerado=DOM_gera_aleatorio(1,100)

          if gerado>=80 then
            minetest.env: add_node (pos, {name = "default:sandstone"})
          elseif gerado >= 40 then
           minetest.env: add_node (pos, {name = "default:desert_stone"})
          else
            minetest.env: add_node (pos, {name = "default:stone"})
          end
        else
          minetest.env: add_node (pos, {name = "lavacooling:moltenrock"})
        end
      end
    end
  end,
})

-- Processo de aquecimento da pedra mole devolta em lava se em contato com lava em mais de um ponto.
minetest.register_abm ({
  nodenames = {"lavacooling:moltenrock","lavacooling:moltenrock_cold"},
  neighbors = {"default:lava_source", "default:lava_flowing"},
  interval = 2.0,
  chance = lavacooling.CHANCE_DE_DERRETER,
  action = function (pos)
    --Sai se rotina estiver desativada.
    if lavacooling.LAVACOOLING_ATIVO==0 then return end

--DOM_inspeciona_r("Inspeção.")

    --Só torna a pedra mole em lava se a fonte de lava estiver com coincidência de dois eixos em ao menos dois pontos, senão ignora.
    --Pegando os 6 pontos chave de contato direto:
    local tipo =  minetest.env:get_node(pos).name


--DOM_inspeciona("Inspeção1:", tipo)

    if tipo == "lavacooling:moltenrock_cold" then
      tipo = "lavacooling:moltenrock"
    else
      tipo="default:lava_flowing"
    end

--DOM_inspeciona("Inspeção2:", tipo)
    local a = DOM_vizinhos(pos)

    if a.total_lava > 1 and a.total_ar < a.total_lava then
      if (a.total_lava_fonte > a.total_lava_corrente) and (tipo=="default:lava_flowing") then
        tipo = "default:lava_source"
      end
      minetest.env: add_node (pos, {name = tipo})
    end
  end,
})

--Converte fontes de lava fluindo em novas fontes de lava se esta estiver entre outras 2 fontes de lava com contato direto.
minetest.register_abm ({
  nodenames = {"default:lava_flowing"},
  neighbors = {"default:lava_source"},
  interval = 2.0,
  chance = 5,
  action = function (pos)
    --Sai se rotina estiver desativada.
    if lavacooling.LAVACOOLING_ATIVO==0 then return end

    local a = DOM_vizinhos(pos)

    if (a.total_lava_fonte > 1) then
      minetest.env: add_node (pos, {name = "default:lava_source"})
    end
  end,
})


--Converte fontes de lava ou lava fluindo em pedra mole se estiver em contato com o ar.
minetest.register_abm ({
  nodenames = {"default:lava_flowing"}, --"default:lava_source"
  neighbors = {"air"},
  interval = 5.0,
  chance = lavacooling.CHANCE_DE_ESFRIAR,
  action = function (pos)
    --Sai se rotina estiver desativada.
    if lavacooling.LAVACOOLING_ATIVO==0 then return end

    local a = DOM_vizinhos(pos)

--[[
  Considera que a lava tem maior chance de esfriar do que de ficar quente então caso o mesmo número de ar e lava forem encontrados a
lava tende a esfriar.
]]--
    if (a.total_ar > 0) and (a.total_lava <= a.total_ar) then
--print("Rotina da lava para pedra mole (rotina da lava).  Lava total e ar:" .. tostring(a.total_lava) .. " " .. tostring(a.total_ar) .. " posição: " .. tostring(pos.x) .. " " .. tostring(pos.y) .. " " .. tostring(pos.z))
      minetest.env: add_node (pos, {name = "lavacooling:moltenrock"})
    end
  end,
})


-- Resfriamento da pedra mole.
minetest.register_abm ({
  nodenames = {"lavacooling:moltenrock"},
  neighbors = {"air"},
  interval = 5.0,
  chance = lavacooling.CHANCE_DE_ESFRIAR,
  action = function (pos)
    --Sai se rotina estiver desativada.
    if lavacooling.LAVACOOLING_ATIVO==0 then return end

    local gerar="lavacooling:moltenrock_cold"
  
    minetest.env: add_node (pos, {name = gerar})
  end,
})


-- Todas as possibilidades de pedra mole fria sobe efeito do ar estão aqui.
minetest.register_abm ({
  nodenames = {"lavacooling:moltenrock_cold"},
  neighbors = {"air"},
  interval = 5.0,
  chance = lavacooling.CHANCE_DE_ESFRIAR,
  action = function (pos)
--DOM_inspeciona_r("Inspeção.")
    --Sai se rotina estiver desativada.
    if lavacooling.LAVACOOLING_ATIVO==0 then return end

    --Testa se será um metal raro:
    local gerado_pedra=DOM_gera_aleatorio(1,100)
    localgerar="default:stone"

    local a = DOM_vizinhos(pos)

--DOM_inspeciona("Chegando a rotina de metais raros/metais, aleatório gerado:",gerado_pedra,a)
--DOM_inspeciona("Chegando a rotina de metais raros/metais, aleatório gerado:",gerado_pedra)
--gerado=100

    if ((gerado_pedra>=lavacooling.COMPARATIVO_PARA_METAIS_RAROS) and (a.total_ar > 0) and (a.total_lava <= a.total_ar)) then
      gerado_pedra=0
      gerado=DOM_gera_aleatorio(1,100)
      gerar="default:stone"
 
--print("ABM do rocha mole (metais raros), aleatorio gerado foi: " .. tostring(gerado))
      local metal_sendo_testado = 1
    
      while gerar=="default:stone" and metal_sendo_testado <= #(lavacooling.METAIS_RAROS) do
        local gerado=DOM_gera_aleatorio(1,100)

        if gerado >= 80 then
          gerar=lavacooling.METAIS_RAROS[metal_sendo_testado]
        else
          metal_sendo_testado=metal_sendo_testado + 1
        end
      end

--DOM_inspeciona("Gerando na rotina metais raros:",gerar)
      if gerar==nil or gerar=="" then
        gerar="default:stone"
      end

    elseif((gerado_pedra>=lavacooling.COMPARATIVO_PARA_METAIS) and (a.total_ar > 0) and (a.total_lava <= a.total_ar)) then
      gerado_pedra=0
--    elseif(gerado>=0) then
      gerar="default:stone"
      gerado=DOM_gera_aleatorio(1,100)
      local metal_sendo_testado = 1
--      print("ABM do rocha mole (metais comuns), aleatorio sorteado foi: " .. tostring(gerado))

--DOM_inspeciona_r("Metais",lavacooling.METAIS)
    
      while gerar=="default:stone" and metal_sendo_testado <= #(lavacooling.METAIS) do
        local gerado=DOM_gera_aleatorio(1,100)
--gerado=90

--DOM_inspeciona("Gerado/Metal:",gerado,metal_sendo_testado)

        if gerado >= 70 then
          gerar=lavacooling.METAIS[metal_sendo_testado]
        else
          metal_sendo_testado=metal_sendo_testado + 1
        end
      end

--DOM_inspeciona("Gerando na rotina metais:",gerar)
      if gerar==nil or gerar=="" then
        gerar="default:stone"
      end
--    elseif ((gerado<lavacooling.COMPARATIVO_PARA_METAIS) and (a.total_ar > 0) and (a.total_lava <= a.total_ar))then
    elseif (a.total_ar > 0) and (a.total_lava <= a.total_ar) then
--DOM_inspeciona_r("ABM do rocha mole (fabrica de pumice e pedras comuns)")

-- Pega o ponto superior e inferior, se tem lava por baixo e ar por cima é produzido pumice.
      if a.n[2]=="air" and (a.n[1]=="default:lava_source" or a.n[1]=="default:lava_flowing") then
        gerar="lavacooling:pumice"
      else
        local gerado=DOM_gera_aleatorio(1,100)

--DOM_inspeciona("Gerando pedras, gerado:",gerado)
        if gerado>=80 then
          gerar="default:sandstone"
        elseif gerado >= 40 then
          gerar="default:desert_stone"
        else
          gerar="default:stone"
        end
      end
    end

    if gerar~="" and gerar ~= nil then
--DOM_inspeciona("Gerando pedras:",gerar)
      --print("Gerando valor da posicao/valor: " .. tostring(gerado) .. "/" .. gerar)
      minetest.env: add_node (pos, {name = gerar})
    end
  end,
})


--Registra comando para configurar o uso do lavacooling
--Comandos previstos:
-- Enable
-- Disable
-- Reload
minetest.register_chatcommand("lavacooling",
{
  params = "",
  description = "Config [DOM]lavacooling mod." ,
  func = function(name,param)
    local j = minetest.env:get_player_by_name(name)
    p = DOM_remove_simbolos(param)
    local p = string.lower(param)

    if p == "enable" then
      default.cool_lava_flowing = lavacooling.cool_lava_flowing_disable
      default.cool_lava_source = lavacooling.cool_lava_source_disable

      lavacooling.LAVACOOLING_ATIVO = 1
      minetest.chat_send_player(name, "Lavacooling is now enabled.")

    elseif p == "disable" then
      lavacooling.LAVACOOLING_ATIVO = 0
      minetest.chat_send_player(name, "Lavacooling is now disabled.")
    elseif p == "reload" then
      local carregados = "0/0"
       
      lavacooling.carrega_minerios()
      carregados = tostring(#(lavacooling.METAIS)).."/"..tostring(#(lavacooling.METAIS_RAROS))
      minetest.chat_send_player(name, "Lavacooling is now reloaded, total of metals/rare metals: "..carregados)
--DOM_inspeciona("Metais carregados:",carregados, lavacooling.METAIS,lavacooling.METAIS_RAROS)
    else
      local estado = "Disabled"
   
      if lavacooling.LAVACOOLING_ATIVO == 1 then
        estado = "Enabled"
      end

      estado = estado .. "(" .. tostring(#(lavacooling.METAIS)).."/"..tostring(#(lavacooling.METAIS_RAROS)) .. ")"
      minetest.chat_send_player(name, "Lavacooling need a param (Enable, Disable or Reload).  Status(metals/rare metals): " .. estado)
    end
  end
})

-- Rotinas quebra-galho:
--[[

--Eliminar ossos de mortes proximo ao jogador
minetest.register_abm ({
  nodenames = {"bones:bones"},
  neighbors = {"default:player"},
  interval = 1.0,
  chance = 1,
  action = function (pos)
--    minetest.env: add_node (pos, {name = "default:water"})
    minetest.env:remove_node(pos)
end,
})

--Eliminar lava proxima ao jogador
minetest.register_abm ({
  nodenames = {"lavacooling:moltenrock", "default:lava_source","default:lava", "default:lava_flowing"},
  neighbors = {"default:player"},
  interval = 1.0,
  chance = 1,
  action = function (pos)
--    minetest.env: add_node (pos, {name = "default:water"})
    minetest.env:remove_node(pos)
end,
})

--Eliminar agua proxima ao jogador
minetest.register_abm ({
  nodenames = {"default:water_source","default:water"},
  neighbors = {"default:player"},
  interval = 1.0,
  chance = 1,
  action = function (pos)
--    minetest.env: add_node (pos, {name = "default:water"})
    minetest.env:remove_node(pos)
end,
})
]]--

lavacooling.carrega_minerios()

-- Apenas para indicar que este módulo foi completamente carregado.
DOM_mb(minetest.get_current_modname(),minetest.get_modpath(minetest.get_current_modname()))
